// SpamWatchClient.h: interface for the SpamWatchClient class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_SpamWatchCLIENT_H__B2F08BC6_1E1C_4952_A9BC_ABC7FBFC1146__INCLUDED_)
#define AFX_SpamWatchCLIENT_H__B2F08BC6_1E1C_4952_A9BC_ABC7FBFC1146__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include <string>
#include <vector>

#define tstring std::basic_string<TCHAR>

/*
  see /doc/protocol.txt for protocol spec.
  */

const int NOT_SPAM = 0;
const int MAYBE_SPAM = 1;
const int SPAM = 2;

const int SPAM_COUNTER_PORT = 3788;

const int REQUEST_CHECK = 1;
const int REQUEST_VOTE = 2;

const unsigned PRIME = 176;
const int FINGERPRINT_COUNT = 10;
const int FINGERPRINTING_LENGTH = 50;

typedef struct {
	long sequence;
	int type;		// REQUEST_CHECK or REQUEST_VOTE
	void *arg;
} spam_request;

class SpamWatchClient  
{
public:
	SpamWatchClient() 
	{
		m_next_seq = 1;

		ptoNComplement = 1;
		int i;
		for (i = 0; i < FINGERPRINTING_LENGTH; i++)
			ptoNComplement = (ptoNComplement * PRIME);
		ptoNComplement = - ptoNComplement;

		m_recv_buf_pos = 0;
	}


	virtual ~SpamWatchClient(){}

	// Connect to the server, return 0 if successful, 
	// return -1 if connection cannot be established
	// result_handler is the handler called when result for checking a text becomes available
	int Connect(LPTSTR host, const int port, void (*result_handler)(int seq, int result, void *arg));

	// Check whether text is the content of a spam email.
	// The text will be normalized before checking.
	// The returned value is a sequence number of the query
	// The sequence number will also be passed to the result_handler when result
	// for this query is available
	// [IN] arg a void * pointer that will be passed to the result handler.
	int CheckText(LPTSTR text, void *arg);

	// Vote for or against the text.  Tell the server whether the specified
	// text is or is not spam.
	// vote can be NOT_SPAM or SPAM
	int Vote(LPTSTR text, const int vote);

	// Block waiting for the replies for all outstanding requests to arrive
	// [IN] timeout timeout in milliseconds
	void Join(const int timeout);
	
	// Wait for all the queries to finish and Closes the connection
	void Close();

private:

	// Remove all redundant space characters from the text
	LPTSTR Normalize(LPTSTR text);

	// Calculate fingerprints for the text.
	// [OUT] fps - pointer to buffer to put fingerprints, should contain 
	int Digest(LPSTR content, unsigned int *fps);

	// Process arrived replies in there is any.  Returns true if any data are received
	bool ProcessReply();
	void SpamWatchClient::ProcessReplyLine(LPSTR line, int len);

	SOCKET m_socket;
	int m_next_seq;
	std::vector <spam_request> m_requests;		// outstanding requests
	
	char m_recv_buf[1024];
	int	m_recv_buf_pos;

	void (*m_result_handler)(int seq, int result, void *arg);
	
	unsigned ptoNComplement;
};

#endif // !defined(AFX_SpamWatchCLIENT_H__B2F08BC6_1E1C_4952_A9BC_ABC7FBFC1146__INCLUDED_)
